@@ -150,7 +150,7 @@ class PtyProcess(object):
         _make_eof_intr()  # Ensure _EOF and _INTR are calculated
         self.pid = pid
         self.fd = fd
-        self.fileobj = self.fileobj_bytes = io.open(fd, 'r+b', buffering=0)
+        self.fileobj = io.open(fd, 'r+b', buffering=0)
 
         self.terminated = False
         self.closed = False
@@ -159,9 +159,6 @@ class PtyProcess(object):
         # status returned by os.waitpid
         self.status = None
         self.flag_eof = False
-        # Delay used before sending data to child. Time in seconds.
-        # Most Linux machines don't like this to be below 0.03 (30 ms).
-        self.delaybeforesend = 0.05
         # Used by close() to give kernel time to update process status.
         # Time in seconds.
         self.delayafterclose = 0.1
@@ -170,7 +167,7 @@ class PtyProcess(object):
         self.delayafterterminate = 0.1
 
     @classmethod
-    def spawn(cls, argv, cwd=None, env=None, echo=True):
+    def spawn(cls, argv, cwd=None, env=None, echo=True, before_exec=()):
         '''Start the given command in a child process in a pseudo terminal.
         
         This does all the fork/exec type of stuff for a pty, and returns an
@@ -184,6 +181,9 @@ class PtyProcess(object):
         # That may not necessarily be bad because you may have spawned a child
         # that performs some task; creates no stdout output; and then dies.
 
+        if not isinstance(argv, (list, tuple)):
+            raise TypeError("Expected a list or tuple for argv, got %r" % argv)
+
         # Shallow copy of argv so we can modify it
         argv = argv[:]
         command = argv[0]
@@ -227,6 +227,10 @@ class PtyProcess(object):
 
             if cwd is not None:
                 os.chdir(cwd)
+
+            for func in before_exec:
+                func()
+
             if env is None:
                 os.execv(command, argv)
             else:
@@ -295,7 +299,7 @@ class PtyProcess(object):
     def fileno(self):
         '''This returns the file descriptor of the pty for the child.
         '''
-        return self.child_fd
+        return self.fd
 
     def close(self, force=True):
         '''This closes the connection with the child application. Note that
@@ -303,16 +307,15 @@ class PtyProcess(object):
         behavior with files. Set force to True if you want to make sure that
         the child is terminated (SIGKILL is sent if the child ignores SIGHUP
         and SIGINT). '''
-
         if not self.closed:
             self.flush()
-            os.close(self.child_fd)
+            self.fileobj.close() # Closes the file descriptor
             # Give kernel time to update process status.
             time.sleep(self.delayafterclose)
             if self.isalive():
                 if not self.terminate(force):
                     raise PtyProcessError('Could not terminate the child.')
-            self.child_fd = -1
+            self.fd = -1
             self.closed = True
             #self.pid = None
 
@@ -331,7 +334,7 @@ class PtyProcess(object):
         methods such as setecho(), setwinsize(), getwinsize() may raise an
         IOError. '''
 
-        return os.isatty(self.child_fd)
+        return os.isatty(self.fd)
 
     def waitnoecho(self, timeout=-1):
         '''This waits until the terminal ECHO flag is set False. This returns
@@ -371,7 +374,7 @@ class PtyProcess(object):
         Not supported on platforms where ``isatty()`` returns False.  '''
 
         try:
-            attr = termios.tcgetattr(self.child_fd)
+            attr = termios.tcgetattr(self.fd)
         except termios.error as err:
             errmsg = 'getecho() may not be called on this platform'
             if err.args[0] == errno.EINVAL:
@@ -473,12 +476,11 @@ class PtyProcess(object):
 
         See also, sendintr() and sendeof().
         '''
-
         char = char.lower()
         a = ord(char)
         if a >= 97 and a <= 122:
             a = a - ord('a') + 1
-            return self.fileobj_bytes.write(_byte(a))
+            return self.fileobj.write(_byte(a))
         d = {'@': 0, '`': 0,
             '[': 27, '{': 27,
             '\\': 28, '|': 28,
@@ -489,7 +491,7 @@ class PtyProcess(object):
         if char not in d:
             return 0
         
-        return self.fileobj_bytes.write(_byte(d[char]))
+        return self.fileobj.write(_byte(d[char]))
 
     def sendeof(self):
         '''This sends an EOF to the child. This sends a character which causes
@@ -501,13 +503,13 @@ class PtyProcess(object):
         It is the responsibility of the caller to ensure the eof is sent at the
         beginning of a line. '''
 
-        self.fileobj_bytes.write(_EOF)
+        self.fileobj.write(_EOF)
 
     def sendintr(self):
         '''This sends a SIGINT to the child. It does not require
         the SIGINT to be the first character on a line. '''
 
-        self.fileobj_bytes.write(_EOF)
+        self.fileobj.write(_EOF)
 
     def eof(self):
         '''This returns True if the EOF exception was ever raised.
